﻿-- Macro Profiles
local MPS_Version = "1.5"

MPS = LibStub("AceAddon-3.0"):NewAddon("Profiles: Macros", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")


local MAX_ACCOUNT_MACROS, MAX_CHARACTER_MACROS = 36, 18

--Char specific macros
minMacros = MAX_ACCOUNT_MACROS + 1
maxMacros = MAX_ACCOUNT_MACROS+MAX_CHARACTER_MACROS

function MPS:OnInitialize()
   MPdb = LibStub("AceDB-3.0"):New("ProfilesMacros",nil,"Default")
end

function MPS:SaveMacroProfile(name)

	MPdb:SetProfile(name)

	print("Profiles: Macros - Saving macros to profile "..name.."!")

	local icons_table = {}

	local numIcons = GetNumMacroIcons()
	for i=1,numIcons do
		local lowercase_icons = string.lower(GetMacroIconInfo(i)) -- Normalize icon path from the API
		icons_table[lowercase_icons] = i
	end
      
	for i=minMacros,maxMacros do
	  
		local name, texture, body = GetMacroInfo(i)
		
		 -- Get icon ID instead of path (needed for CreateMacro() in Load!
		texture = string.lower(texture)
        if icons_table[texture] then 
             texture = icons_table[texture]
        end
		
		-- Icon not found, give question mark!
		if (type(texture) ~= "number") then 
		texture = '1'  end
		  
		-- Save the macro to db
		MPdb.profile[i] = { name,texture,body }
	end   
end

function MPS:LoadMacroProfile(name)
   
   -- Create an empty profile if it does not exist
    MPdb:SetProfile(name)
   
   print("Loading!")
   
   --TODO: Check actionbars in case we have macros that will be removed
   
   --Get rid of all the current macros
   for i=minMacros,maxMacros do
      DeleteMacro(minMacros) -- DeleteMacro changes index for all other macros
   end 
   
   --Load saved macros profile
   for index, keys in pairs(MPdb.profile) do   
       if keys[1] then
            
            if (type(keys[2]) ~= "number") then keys[2] = '1' end -- Icon not found, give question mark!
            
            CreateMacro(keys[1], keys[2], keys[3], 1);
       end
   end
   
end


function MPS:ExistsProfile(name)
   for i, k in ipairs(MPdb:GetProfiles()) do
      local key = k
      
      if key == name then
         return (key)
      end
   end
   return nil
end


SLASH_MP1 = "/mp"
SlashCmdList["MP"] = function(msg)
   local cmd, arg = string.split(" ", msg, 2)
   cmd = string.lower(cmd or "")
   arg = arg or ""
   
   if (cmd == "version") then
      print("Profiles: Macros v."..MPS_Version)
   elseif (cmd=="list")then
      for i,k in ipairs(MPdb:GetProfiles()) do
		if k == MPdb:GetCurrentProfile() then
			print(' - ' .. k.." (current)")
		else
			print(' - ' .. k)
		end
	end
   elseif (cmd=="" or arg=="")then
      print("/mp delete <name> * Not implemepted yet")
      print("/mp save <name>")
      print("/mp load <name>")
      print("/mp list")
      print("/mp version")
      print("/mp debug [on|off] * Not implemepted yet")
   elseif (cmd=="save" and arg~="")then
      MPS:SaveMacroProfile(arg)
   elseif (cmd=="load" and arg~="")then
      MPS:LoadMacroProfile(arg)
   elseif (cmd=="delete" and arg~="")then
	print("Profiles - Macros: Not implemented yet!")
   else    
      print("wrong entry")
   end
end

--Regexp routine
-- nocase("Hi there!")) >>> 
-- [hH][iI] [tT][hH][eE][rR][eE]!
function MPS:nocase (s)
   s = string.gsub(s, "%a", function (c)
         return string.format("[%s%s]", string.lower(c),
            string.upper(c))
   end)
   return s
end